@@ -25,15 +25,16 @@ import (
 	"syscall"
 	"time"
 
-	"github.com/elazarl/go-bindata-assetfs"
 	"github.com/julienschmidt/httprouter"
 	"github.com/prometheus/client_golang/prometheus"
+	"github.com/prometheus/client_golang/prometheus/promhttp"
 	"github.com/prometheus/common/log"
 	"github.com/prometheus/common/version"
 	"gopkg.in/alecthomas/kingpin.v2"
 
 	dto "github.com/prometheus/client_model/go"
 
+	"github.com/prometheus/pushgateway/asset"
 	"github.com/prometheus/pushgateway/handler"
 	"github.com/prometheus/pushgateway/storage"
 )
@@ -72,7 +73,8 @@ func main() {
 	for _, f := range app.Model().Flags {
 		flags[f.Name] = f.Value.String()
 	}
-	ms := storage.NewDiskMetricStore(*persistenceFile, *persistenceInterval)
+
+	ms := storage.NewDiskMetricStore(*persistenceFile, *persistenceInterval, prometheus.DefaultGatherer)
 
 	// Inject the metric families returned by ms.GetMetricFamilies into the default Gatherer:
 	prometheus.DefaultGatherer = prometheus.Gatherers{
@@ -81,10 +83,9 @@ func main() {
 	}
 
 	r := httprouter.New()
-	r.Handler("GET", *routePrefix+"/-/healthy", prometheus.InstrumentHandlerFunc("healthy", handler.Healthy(ms)))
-	r.Handler("GET", *routePrefix+"/-/ready", prometheus.InstrumentHandlerFunc("ready", handler.Ready(ms)))
-
-	r.Handler("GET", path.Join(*routePrefix, *metricsPath), prometheus.Handler())
+	r.Handler("GET", *routePrefix+"/-/healthy", handler.Healthy(ms))
+	r.Handler("GET", *routePrefix+"/-/ready", handler.Ready(ms))
+	r.Handler("GET", path.Join(*routePrefix, *metricsPath), promhttp.Handler())
 
 	// Handlers for pushing and deleting metrics.
 	pushAPIPath := *routePrefix + "/metrics"
@@ -95,26 +96,9 @@ func main() {
 	r.POST(pushAPIPath+"/job/:job", handler.Push(ms, false))
 	r.DELETE(pushAPIPath+"/job/:job", handler.Delete(ms))
 
-	// Handlers for the deprecated API.
-	r.PUT(pushAPIPath+"/jobs/:job/instances/:instance", handler.LegacyPush(ms, true))
-	r.POST(pushAPIPath+"/jobs/:job/instances/:instance", handler.LegacyPush(ms, false))
-	r.DELETE(pushAPIPath+"/jobs/:job/instances/:instance", handler.LegacyDelete(ms))
-	r.PUT(pushAPIPath+"/jobs/:job", handler.LegacyPush(ms, true))
-	r.POST(pushAPIPath+"/jobs/:job", handler.LegacyPush(ms, false))
-	r.DELETE(pushAPIPath+"/jobs/:job", handler.LegacyDelete(ms))
+	r.Handler("GET", *routePrefix+"/static/*filepath", handler.Static(asset.Assets))
 
-	r.Handler("GET", *routePrefix+"/static/*filepath", prometheus.InstrumentHandler(
-		"static",
-		http.FileServer(
-			&assetfs.AssetFS{
-				Asset: func(name string) ([]byte, error) {
-					path := name[len(*routePrefix):]
-					return Asset(path)
-				},
-				AssetDir: AssetDir, AssetInfo: AssetInfo},
-		),
-	))
-	statusHandler := prometheus.InstrumentHandlerFunc("status", handler.Status(ms, Asset, flags))
+	statusHandler := handler.Status(ms, asset.Assets, flags)
 	r.Handler("GET", *routePrefix+"/status", statusHandler)
 	r.Handler("GET", *routePrefix+"/", statusHandler)
 
