@@ -24,6 +24,7 @@ import (
 	"time"
 
 	"github.com/golang/protobuf/proto"
+	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/common/model"
 
 	dto "github.com/prometheus/client_model/go"
@@ -283,6 +284,138 @@ var (
 			},
 		},
 	}
+	mfh1 = &dto.MetricFamily{
+		Name: proto.String("mf_help"),
+		Help: proto.String("Help string for mfh1."),
+		Type: dto.MetricType_GAUGE.Enum(),
+		Metric: []*dto.Metric{
+			{
+				Label: []*dto.LabelPair{
+					{
+						Name:  proto.String("job"),
+						Value: proto.String("job1"),
+					},
+				},
+				Gauge: &dto.Gauge{
+					Value: proto.Float64(3948.838),
+				},
+			},
+		},
+	}
+	mfh2 = &dto.MetricFamily{
+		Name: proto.String("mf_help"),
+		Help: proto.String("Help string for mfh2."),
+		Type: dto.MetricType_GAUGE.Enum(),
+		Metric: []*dto.Metric{
+			{
+				Label: []*dto.LabelPair{
+					{
+						Name:  proto.String("job"),
+						Value: proto.String("job2"),
+					},
+				},
+				Gauge: &dto.Gauge{
+					Value: proto.Float64(83),
+				},
+			},
+		},
+	}
+	// Both mfh metrics with mfh1's help string.
+	mfh12 = &dto.MetricFamily{
+		Name: proto.String("mf_help"),
+		Help: proto.String("Help string for mfh1."),
+		Type: dto.MetricType_GAUGE.Enum(),
+		Metric: []*dto.Metric{
+			{
+				Label: []*dto.LabelPair{
+					{
+						Name:  proto.String("job"),
+						Value: proto.String("job1"),
+					},
+				},
+				Gauge: &dto.Gauge{
+					Value: proto.Float64(3948.838),
+				},
+			},
+			{
+				Label: []*dto.LabelPair{
+					{
+						Name:  proto.String("job"),
+						Value: proto.String("job2"),
+					},
+				},
+				Gauge: &dto.Gauge{
+					Value: proto.Float64(83),
+				},
+			},
+		},
+	}
+	// Both mfh metrics with mfh2's help string.
+	mfh21 = &dto.MetricFamily{
+		Name: proto.String("mf_help"),
+		Help: proto.String("Help string for mfh2."),
+		Type: dto.MetricType_GAUGE.Enum(),
+		Metric: []*dto.Metric{
+			{
+				Label: []*dto.LabelPair{
+					{
+						Name:  proto.String("job"),
+						Value: proto.String("job1"),
+					},
+				},
+				Gauge: &dto.Gauge{
+					Value: proto.Float64(3948.838),
+				},
+			},
+			{
+				Label: []*dto.LabelPair{
+					{
+						Name:  proto.String("job"),
+						Value: proto.String("job2"),
+					},
+				},
+				Gauge: &dto.Gauge{
+					Value: proto.Float64(83),
+				},
+			},
+		},
+	}
+	mfgg = &dto.MetricFamily{
+		Name: proto.String("go_goroutines"),
+		Help: proto.String("Inconsistent doc string, fixed version in mfggFixed."),
+		Type: dto.MetricType_GAUGE.Enum(),
+		Metric: []*dto.Metric{
+			{
+				Label: []*dto.LabelPair{
+					{
+						Name:  proto.String("job"),
+						Value: proto.String("job1"),
+					},
+				},
+				Gauge: &dto.Gauge{
+					Value: proto.Float64(5),
+				},
+			},
+		},
+	}
+	mfggFixed = &dto.MetricFamily{
+		Name: proto.String("go_goroutines"),
+		Help: proto.String("Number of goroutines that currently exist."),
+		Type: dto.MetricType_GAUGE.Enum(),
+		Metric: []*dto.Metric{
+			{
+				Label: []*dto.LabelPair{
+					{
+						Name:  proto.String("job"),
+						Value: proto.String("job1"),
+					},
+				},
+				Gauge: &dto.Gauge{
+					Value: proto.Float64(5),
+				},
+			},
+		},
+	}
 )
 
 func addGroup(
@@ -390,7 +523,7 @@ func TestAddDeletePersistRestore(t *testing.T) {
 	}
 	defer os.RemoveAll(tempDir)
 	fileName := path.Join(tempDir, "persistence")
-	dms := NewDiskMetricStore(fileName, 100*time.Millisecond)
+	dms := NewDiskMetricStore(fileName, 100*time.Millisecond, nil)
 
 	// Submit a single simple metric family.
 	ts1 := time.Now()
@@ -458,7 +591,7 @@ func TestAddDeletePersistRestore(t *testing.T) {
 	}
 
 	// Load it again.
-	dms = NewDiskMetricStore(fileName, 100*time.Millisecond)
+	dms = NewDiskMetricStore(fileName, 100*time.Millisecond, nil)
 	if err := checkMetricFamilies(dms, mf1a, mf2, mf3, mf5); err != nil {
 		t.Error(err)
 	}
@@ -555,7 +688,7 @@ func TestAddDeletePersistRestore(t *testing.T) {
 }
 
 func TestNoPersistence(t *testing.T) {
-	dms := NewDiskMetricStore("", 100*time.Millisecond)
+	dms := NewDiskMetricStore("", 100*time.Millisecond, nil)
 
 	ts1 := time.Now()
 	dms.SubmitWriteRequest(WriteRequest{
@@ -575,7 +708,7 @@ func TestNoPersistence(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	dms = NewDiskMetricStore("", 100*time.Millisecond)
+	dms = NewDiskMetricStore("", 100*time.Millisecond, nil)
 	if err := checkMetricFamilies(dms); err != nil {
 		t.Error(err)
 	}
@@ -589,48 +722,65 @@ func TestNoPersistence(t *testing.T) {
 	}
 }
 
-func TestLegacyRestore(t *testing.T) {
-	tempDir, err := ioutil.TempDir("", "diskmetricstore.TestLegacyRestore.")
-	if err != nil {
-		t.Fatal(err)
-	}
-	defer os.RemoveAll(tempDir)
-	fileName := path.Join(tempDir, "persistence")
-	dms := NewDiskMetricStore(fileName, 100*time.Millisecond)
+func TestHelpStringFix(t *testing.T) {
+	dms := NewDiskMetricStore("", 100*time.Millisecond, prometheus.DefaultGatherer)
 
-	// Submit a single simple metric family.
 	ts1 := time.Now()
 	dms.SubmitWriteRequest(WriteRequest{
 		Labels: map[string]string{
-			"job":      "job1",
-			"instance": "instance1",
+			"job": "job1",
+		},
+		Timestamp: ts1,
+		MetricFamilies: map[string]*dto.MetricFamily{
+			"go_goroutines": mfgg,
+			"mf_help":       mfh1,
+		},
+	})
+	dms.SubmitWriteRequest(WriteRequest{
+		Labels: map[string]string{
+			"job": "job2",
+		},
+		Timestamp: ts1,
+		MetricFamilies: map[string]*dto.MetricFamily{
+			"mf_help": mfh2,
 		},
-		Timestamp:      ts1,
-		MetricFamilies: map[string]*dto.MetricFamily{"mf3": mf3},
 	})
 	time.Sleep(20 * time.Millisecond) // Give loop() time to process.
-	if err := checkMetricFamilies(dms, mf3); err != nil {
-		t.Error(err)
+
+	// Either we have settle on the mfh1 help string or the mfh2 help string.
+	gotMFs := dms.GetMetricFamilies()
+	if len(gotMFs) != 2 {
+		t.Fatalf("expected 2 metric families, got %d", len(gotMFs))
+	}
+	gotMFsAsStrings := make([]string, len(gotMFs))
+	for i, mf := range gotMFs {
+		sort.Sort(metricSorter(mf.GetMetric()))
+		gotMFsAsStrings[i] = mf.String()
+	}
+	sort.Strings(gotMFsAsStrings)
+	gotGG := gotMFsAsStrings[0]
+	got12 := gotMFsAsStrings[1]
+	expectedGG := mfggFixed.String()
+	expected12 := mfh12.String()
+	expected21 := mfh21.String()
+
+	if gotGG != expectedGG {
+		t.Errorf(
+			"help strings weren't properly adjusted, got '%s', expected '%s'",
+			gotGG, expectedGG,
+		)
+	}
+	if got12 != expected12 && got12 != expected21 {
+		t.Errorf(
+			"help strings weren't properly adjusted, got '%s' which is neither '%s' nor '%s'",
+			got12, expected12, expected21,
+		)
 	}
 
-	// Manipulate dms internals to simulate the legacy persistence format.
-	for _, mg := range dms.metricGroups {
-		tmf := mg.Metrics["mf3"]
-		tmf.MetricFamily = (*dto.MetricFamily)(tmf.GobbableMetricFamily)
-		tmf.GobbableMetricFamily = nil
-		mg.Metrics["mf3"] = tmf
-	}
-
-	// Shutdown the dms to persist in legacy format.
 	if err := dms.Shutdown(); err != nil {
 		t.Fatal(err)
 	}
 
-	// Load it again.
-	dms = NewDiskMetricStore(fileName, 100*time.Millisecond)
-	if err := checkMetricFamilies(dms, mf3); err != nil {
-		t.Error(err)
-	}
 }
 
 func checkMetricFamilies(dms *DiskMetricStore, expectedMFs ...*dto.MetricFamily) error {
@@ -656,7 +806,7 @@ func checkMetricFamilies(dms *DiskMetricStore, expectedMFs ...*dto.MetricFamily)
 	for i, got := range gotMFsAsStrings {
 		expected := expectedMFsAsStrings[i]
 		if expected != got {
-			return fmt.Errorf("expected metric family %#v, got %#v", expected, got)
+			return fmt.Errorf("expected metric family '%s', got '%s'", expected, got)
 		}
 	}
 	return nil
